<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * map v 1.0
 * 
 * класс работает с структурой сайта (роутинг)
 *
 */

class map
{   
    /*
     * хранит значение посланные в метод __call()
     *
     * @var    - массив
     * @access - private
     */
    
    private $call = array();
    
    /*
     * массив c страницами
     *
     * @var    - array
     * @access - public
     */
    
    public $pageList;
    
    /*
     * метод позволяем вызвать несуществующие методы класса
     * 
     * @access - public
     * @return - сам себя или сохраненное значение (все зависит от
     *           того set или get метод был вызван)
     * 
     * @param string name      - имя вызываемого метода
     * @param array  arguments - присланные аргументы в вызываемый метод
     */
    
    public function __call($name, $arguments)
    {
        $name       = mb_strtolower($name);
        
        $action     = mb_substr($name, 0, 3);
        $argument   = mb_substr($name, 3, mb_strlen($name)-1);
        
        switch($action) {
            case 'get':
                return (isset($this->call[$argument])) ? $this->call[$argument] : null;
                break;
            case 'set':
                $this->call[$argument] = $arguments[0];

                return $this;
                break;
        }
    }
    
    /*
     * получаем шаблон, все контроллеры и их параметры открываемой страницы
     * 
     * @access - public
     * @return - массив с данными или false если ничего не найдено
     */
     
    public function getPage()
    {
        // получаем адрес страницы
        $uri = request::getHttpServer('REQUEST_URI')->toUri();
        
        // если адрес равен null выкидываем false
        if (is_null($uri))
            return FALSE;

        // просто бьем строку на массив
        $uri = explode('/', $uri);

        // удаляем последний элемент массива, так как он всегда пустой
        unset($uri[(count($uri) - 1)]); 
            
        // получаем все открываемые узлы
        // родитель по умолчанию NULL (главная страница)
        $parent = NULL;
        foreach ($uri as $key => $alias) {
            // получаем страницу
            if (!$page = $this->setAlias($alias)->setParent($parent)->getNode())
                return FALSE;
            
            // запоминаем ID страницы, это наш родитель в следующей итерации
            $parent = $page['map_id'];

            // создаем объект страницы
            $this->pageList[] = new page($page);
        }
        
        return TRUE;
    }

    /*
     * метод получения узла определенной ветки
     * 
     * @access - private
     * @return - массив с структурой сайта или false если ничего не найдено
     * 
     * @param integer getParent() - родитель предшествующего узла 
     * @param string  getAlias()  - алиас узла 
     */
     
    private function getNode()
    {
        // с начало пытаемся получить страницу как статичную
        $sql = ''
            . 'SELECT'
                . ' m.map_id            AS map_id,'
                . ' m.map_parent        AS map_parent,'
                . ' m.map_alias         AS map_alias,'
                . ' m.map_name          AS map_name,'
                . ' m.map_view          AS map_view,'
                . ' m.map_system        AS map_system,'
                . ' m.map_type          AS map_type'
            . ' FROM'
                . ' rs_map              AS m'
            . ' WHERE'
                . '     m.map_parent    ' . ( (is_null($this->getParent())) ? 'IS NULL' : '= \'' . $this->getParent() . '\'' )
                . ' AND m.map_alias     ' . ( ($this->getAlias() == '')     ? 'IS NULL' : '= \'' . $this->getAlias() . '\'' )
                . ' AND m.map_type      = \'static\''
        ;
        
        // если выборка дала результат
        if ($resultList = sql::query($sql)->fetch()) {
            // запоминаем алиас динамичного узла.
            // так как узел статичный, то мы записываем туда NULL
            $resultList[0]['dynamic_alias'] = NULL;
            // возвращаем результат
            return $resultList[0];            
        }
        
        // если статичный узел был не найден, пытаемся найти динамичный
        $sql = ''
            . 'SELECT'
                . ' m.map_id            AS map_id,'
                . ' m.map_parent        AS map_parent,'
                . ' m.map_alias         AS map_alias,'
                . ' m.map_name          AS map_name,'
                . ' m.map_view          AS map_view,'
                . ' m.map_system        AS map_system,'
                . ' m.map_type          AS map_type'
            . ' FROM'
                . ' rs_map              AS m'
            . ' WHERE'
                . '     m.map_parent    ' . ( (is_null($this->getParent())) ? 'IS NULL' : '= \'' . $this->getParent() . '\'' )
                . ' AND m.map_type      = \'dynamic\''
        ;
        
        // если выборка не дала результата
        if (!$resultList = sql::query($sql)->fetch())
            // вертаем false
            return FALSE;
            
        // запоминаем динамичный алиас (реально записанный в БД)
        $resultList[0]['dynamic_alias'] = $resultList[0]['map_alias'];
        // перезаписываем псевдоним на тот что в адресной строке
        $resultList[0]['map_alias']     = $this->getAlias();
            
        // возвращаем результат
        return $resultList[0];        
    }
}
